import Vue from 'vue'
import Vuex from 'vuex'

Vue.use(Vuex)

export default new Vuex.Store({
    state: {
        ws_connected: false,
        notifications: [],
        scan_cache: {}
    },
    getters: {
        getScansByTarget(state, target){
            return state.scan_cache[target]
        }
    },
    actions: {
        getScansByTarget({ commit }, target){
            return fetch('/api/v1/scans/' + target)
                .then(resp => resp.json())
                .then(json => {
                    commit("SET_TARGET_SCANS", { target, data: json.data });
                });
        },
        performScan({ commit }, target) {
            return fetch('/api/v1/scans', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({ "target": target })
            }).then(async resp => {
                let scan = await resp.json()
                commit("UPDATE_SCAN", { scan });
                return scan;
            });
        }
    },
    mutations: {
        "SET_TARGET_SCANS"(state, { target, data }) {
            Vue.set(state.scan_cache, target, data)
        },
        "UPDATE_SCAN"(state, { scan }) {
            let target = scan.target;
            if (!state.scan_cache[target]) {
                Vue.set(state.scan_cache, target, [scan])
            } else {
                let matchedItem = state.scan_cache[target]
                    .find(item => item.id == scan.id);

                if (matchedItem)
                    Object.keys(scan).forEach(key => {
                        Vue.set(matchedItem, key, scan[key])
                    });
                else
                    state.scan_cache[target].unshift(scan);
            }
        },
        "SOCKET_message"(state, scan) {
            // Update progress queue
            let matchedItem = state.notifications.find(item => item.id == scan.id);
            if (matchedItem)
                Object.keys(scan).forEach(key => {
                    Vue.set(matchedItem, key, scan[key])
                });
            else
                state.notifications.push(scan);

            // Update scan cache
            this.commit("UPDATE_SCAN", { scan });
        },
        "SOCKET_connect"(state) {
            state.ws_connected = true
        },
        "SOCKET_disconnect"(state) {
            state.ws_connected = false
        },
    },
})
